(**************************************************************************)
(*                                                                        *)
(*  FORK                                                                  *)
(*                                                                        *)
(*  Author:  Franois Pottier, INRIA Paris-Rocquencourt                   *)
(*  Version: 20101109                                                     *)
(*                                                                        *)
(*  The copyright to this code is held by Institut National de Recherche  *)
(*  en Informatique et en Automatique (INRIA). All rights reserved. This  *)
(*  file is distributed under the license CeCILL-C (see file LICENSE).    *)
(*                                                                        *)
(**************************************************************************)

open Atom

(* The kinds in FORK correspond to the types in Nakano's type system. [KStar]
   is the base kind: it classifies types that classify values. [KDefined] is a
   reference to a user-defined kind. [KArrow] is the arrow kind
   constructor. [KLater] is Nakano's modality. *)

type kind =
  | KStar
  | KDefined of atom
  | KArrow of kind * kind
  | KLater of kind

(* Kinds can be (recursively) defined by the user. A (global) kind
   signature maps atoms to kinds. *)

type signature =
    kind AtomMap.t

(* Syntactic equality of kinds. *)

val equal: kind -> kind -> bool

(* This function installs a kind signature and checks that every kind is
   well-formed and tail-finite. It must be called exactly once. *)

val install: signature -> unit

(* Displaying kinds. *)

val print_kind: kind -> string

(* Subkind checking. *)

val is_subkind: kind -> kind -> bool
val is_subkind_later: kind -> kind -> bool

(* [kapp locs kind1 kind2] determines what kind is obtained when an object of
   kind [kind1] is applied to an object of kind [kind2]. [KindChecking] is
   raised if such an application is invalid. *)

exception KindChecking of Error.location list * string

val kapp: Error.location list -> kind -> kind -> kind

(* Determine whether a kind is of the form [later^k *]. *)

val is_later_star: kind -> bool

